/****************************************************************************
** Meta object code from reading C++ file 'fcitx5configproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../deepin-fcitx5configtool-plugin/src/dcc-fcitx5configtool/operation/fcitx5configproxy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fcitx5configproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6deepin16fcitx5configtool17Fcitx5ConfigProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto deepin::fcitx5configtool::Fcitx5ConfigProxy::qt_create_metaobjectdata<qt_meta_tag_ZN6deepin16fcitx5configtool17Fcitx5ConfigProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "deepin::fcitx5configtool::Fcitx5ConfigProxy",
        "requestConfigFinished",
        "",
        "saveTriggeredChanged",
        "value",
        "QVariant",
        "path",
        "setValue",
        "isKey",
        "save",
        "globalConfigTypes",
        "QVariantList",
        "globalConfigOptions",
        "type",
        "all",
        "globalConfigOption",
        "option",
        "restoreDefault",
        "requestConfig",
        "sync",
        "fromSave",
        "onRequestConfigFinished",
        "QDBusPendingCallWatcher*",
        "watcher",
        "saveTriggered"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestConfigFinished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saveTriggeredChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'value'
        QtMocHelpers::SlotData<QVariant(const QString &) const>(4, 2, QMC::AccessPublic, 0x80000000 | 5, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'setValue'
        QtMocHelpers::SlotData<void(const QString &, const QVariant &, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 5, 4 }, { QMetaType::Bool, 8 },
        }}),
        // Slot 'setValue'
        QtMocHelpers::SlotData<void(const QString &, const QVariant &)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 5, 4 },
        }}),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'globalConfigTypes'
        QtMocHelpers::SlotData<QVariantList() const>(10, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'globalConfigOptions'
        QtMocHelpers::SlotData<QVariantList(const QString &, bool) const>(12, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::QString, 13 }, { QMetaType::Bool, 14 },
        }}),
        // Slot 'globalConfigOptions'
        QtMocHelpers::SlotData<QVariantList(const QString &) const>(12, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 11, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'globalConfigOption'
        QtMocHelpers::SlotData<QVariant(const QString &, const QString &) const>(15, 2, QMC::AccessPublic, 0x80000000 | 5, {{
            { QMetaType::QString, 13 }, { QMetaType::QString, 16 },
        }}),
        // Slot 'restoreDefault'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'requestConfig'
        QtMocHelpers::SlotData<void(bool, bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'requestConfig'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Slot 'onRequestConfigFinished'
        QtMocHelpers::SlotData<void(QDBusPendingCallWatcher *)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'saveTriggered'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Fcitx5ConfigProxy, qt_meta_tag_ZN6deepin16fcitx5configtool17Fcitx5ConfigProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject deepin::fcitx5configtool::Fcitx5ConfigProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6deepin16fcitx5configtool17Fcitx5ConfigProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6deepin16fcitx5configtool17Fcitx5ConfigProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6deepin16fcitx5configtool17Fcitx5ConfigProxyE_t>.metaTypes,
    nullptr
} };

void deepin::fcitx5configtool::Fcitx5ConfigProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Fcitx5ConfigProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestConfigFinished(); break;
        case 1: _t->saveTriggeredChanged(); break;
        case 2: { QVariant _r = _t->value((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->setValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 4: _t->setValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 5: _t->save(); break;
        case 6: { QVariantList _r = _t->globalConfigTypes();
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 7: { QVariantList _r = _t->globalConfigOptions((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 8: { QVariantList _r = _t->globalConfigOptions((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 9: { QVariant _r = _t->globalConfigOption((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->restoreDefault((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->requestConfig((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 12: _t->requestConfig((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->onRequestConfigFinished((*reinterpret_cast<std::add_pointer_t<QDBusPendingCallWatcher*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Fcitx5ConfigProxy::*)()>(_a, &Fcitx5ConfigProxy::requestConfigFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fcitx5ConfigProxy::*)()>(_a, &Fcitx5ConfigProxy::saveTriggeredChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->saveTriggered(); break;
        default: break;
        }
    }
}

const QMetaObject *deepin::fcitx5configtool::Fcitx5ConfigProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *deepin::fcitx5configtool::Fcitx5ConfigProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6deepin16fcitx5configtool17Fcitx5ConfigProxyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int deepin::fcitx5configtool::Fcitx5ConfigProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void deepin::fcitx5configtool::Fcitx5ConfigProxy::requestConfigFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void deepin::fcitx5configtool::Fcitx5ConfigProxy::saveTriggeredChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
