/****************************************************************************
** Meta object code from reading C++ file 'fcitx5configtool.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../deepin-fcitx5configtool-plugin/src/dcc-fcitx5configtool/operation/fcitx5configtool.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fcitx5configtool.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6deepin16fcitx5configtool22Fcitx5ConfigToolWorkerE_t {};
} // unnamed namespace

template <> constexpr inline auto deepin::fcitx5configtool::Fcitx5ConfigToolWorker::qt_create_metaobjectdata<qt_meta_tag_ZN6deepin16fcitx5configtool22Fcitx5ConfigToolWorkerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "deepin::fcitx5configtool::Fcitx5ConfigToolWorker",
        "requestConfigFinished",
        "",
        "requestAddonsFinished",
        "init",
        "showAddonSettingsDialog",
        "addonStr",
        "title",
        "imlistModel",
        "IMListModel*",
        "imProxyModel",
        "fcitx::kcm::IMProxyModel*",
        "openDeepinAppStore",
        "showIMSettingsDialog",
        "index",
        "addIM",
        "launchConfigDialog",
        "uri",
        "fcitx5ConfigProxy",
        "Fcitx5ConfigProxy*",
        "fcitx5AddonsProxy",
        "Fcitx5AddonsProxy*",
        "keyboardController",
        "dccV25::KeyboardController*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestConfigFinished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestAddonsFinished'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'init'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Method 'showAddonSettingsDialog'
        QtMocHelpers::MethodData<void(const QString &, const QString &) const>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 },
        }}),
        // Method 'imlistModel'
        QtMocHelpers::MethodData<IMListModel *() const>(8, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Method 'imProxyModel'
        QtMocHelpers::MethodData<fcitx::kcm::IMProxyModel *() const>(10, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Method 'openDeepinAppStore'
        QtMocHelpers::MethodData<void() const>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'showIMSettingsDialog'
        QtMocHelpers::MethodData<void(int) const>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Method 'addIM'
        QtMocHelpers::MethodData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Method 'launchConfigDialog'
        QtMocHelpers::MethodData<void(const QString &, const QString &) const>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'fcitx5ConfigProxy'
        QtMocHelpers::PropertyData<Fcitx5ConfigProxy*>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'fcitx5AddonsProxy'
        QtMocHelpers::PropertyData<Fcitx5AddonsProxy*>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'keyboardController'
        QtMocHelpers::PropertyData<dccV25::KeyboardController*>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Fcitx5ConfigToolWorker, qt_meta_tag_ZN6deepin16fcitx5configtool22Fcitx5ConfigToolWorkerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject deepin::fcitx5configtool::Fcitx5ConfigToolWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6deepin16fcitx5configtool22Fcitx5ConfigToolWorkerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6deepin16fcitx5configtool22Fcitx5ConfigToolWorkerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6deepin16fcitx5configtool22Fcitx5ConfigToolWorkerE_t>.metaTypes,
    nullptr
} };

void deepin::fcitx5configtool::Fcitx5ConfigToolWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Fcitx5ConfigToolWorker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestConfigFinished(); break;
        case 1: _t->requestAddonsFinished(); break;
        case 2: _t->init(); break;
        case 3: _t->showAddonSettingsDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: { IMListModel* _r = _t->imlistModel();
            if (_a[0]) *reinterpret_cast<IMListModel**>(_a[0]) = std::move(_r); }  break;
        case 5: { fcitx::kcm::IMProxyModel* _r = _t->imProxyModel();
            if (_a[0]) *reinterpret_cast<fcitx::kcm::IMProxyModel**>(_a[0]) = std::move(_r); }  break;
        case 6: _t->openDeepinAppStore(); break;
        case 7: _t->showIMSettingsDialog((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->addIM((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->launchConfigDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Fcitx5ConfigToolWorker::*)()>(_a, &Fcitx5ConfigToolWorker::requestConfigFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Fcitx5ConfigToolWorker::*)()>(_a, &Fcitx5ConfigToolWorker::requestAddonsFinished, 1))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Fcitx5AddonsProxy* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Fcitx5ConfigProxy* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Fcitx5ConfigProxy**>(_v) = _t->fcitx5ConfigProxy(); break;
        case 1: *reinterpret_cast<Fcitx5AddonsProxy**>(_v) = _t->fcitx5AddonsProxy(); break;
        case 2: *reinterpret_cast<dccV25::KeyboardController**>(_v) = _t->keyboardController(); break;
        default: break;
        }
    }
}

const QMetaObject *deepin::fcitx5configtool::Fcitx5ConfigToolWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *deepin::fcitx5configtool::Fcitx5ConfigToolWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6deepin16fcitx5configtool22Fcitx5ConfigToolWorkerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int deepin::fcitx5configtool::Fcitx5ConfigToolWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void deepin::fcitx5configtool::Fcitx5ConfigToolWorker::requestConfigFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void deepin::fcitx5configtool::Fcitx5ConfigToolWorker::requestAddonsFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
